@@ -9,7 +9,7 @@ use Inline (
         C => "DATA",
         LIBS => '-lusb',
 	NAME => 'Device::USB',
-	VERSION => '0.16',
+	VERSION => '0.18',
    );
 
 Inline->init();
@@ -19,19 +19,30 @@ Inline->init();
 #
 
 use Device::USB::Device;
+use Device::USB::DevConfig;
 use Device::USB::Bus;
 
+use constant CLASS_PER_INSTANCE => 0;
+use constant CLASS_AUDIO => 1;
+use constant CLASS_COMM =>  2;
+use constant CLASS_HID =>   3;
+use constant CLASS_PRINTER => 7;
+use constant CLASS_MASS_STORAGE => 8;
+use constant CLASS_HUB =>     9;
+use constant CLASS_DATA =>   10;
+use constant CLASS_VENDOR_SPEC => 0xff;
+
 =head1 NAME
 
 Device::USB - Use libusb to access USB devices.
 
 =head1 VERSION
 
-Version 0.16
+Version 0.18
 
 =cut
 
-our $VERSION='0.16';
+our $VERSION='0.18';
 
 
 =head1 SYNOPSIS
@@ -98,6 +109,51 @@ functions of the libusb library. In particular, it provides interfaces to
 find busses and devices. It also provides convenience methods that simplify
 some of the tasks above.
 
+=head2 CONSTANTS
+
+This class provides a set of constants for the defined device classes. The
+constants defined at this time are:
+
+=over 4
+
+=item *
+
+CLASS_PER_INSTANCE
+
+=item *
+
+CLASS_AUDIO
+
+=item *
+
+CLASS_COMM
+
+=item *
+
+CLASS_HID
+
+=item *
+
+CLASS_PRINTER
+
+=item *
+
+CLASS_MASS_STORAGE
+
+=item *
+
+CLASS_HUB
+
+=item *
+
+CLASS_DATA
+
+=item *
+
+CLASS_VENDOR_SPEC
+
+=back
+
 =head2 FUNCTIONS
 
 =over 4
@@ -135,7 +191,7 @@ interface code.
 
 =over 4
 
-=item $level
+=item level
 
 0 disables debugging, 1 enables some debug messages, and 2 enables verbose
 debug messages
@@ -183,9 +239,9 @@ sub find_devices
 
 =item find_device
 
-Find and a particular USB device based on the vendor and product ids. If more
-than one device has the same product id from the same vendor, the fist one
-found will be returned.
+Find a particular USB device based on the vendor and product ids. If more
+than one device has the same product id from the same vendor, the first one
+found is returned.
 
 =over 4
 
@@ -210,6 +266,44 @@ sub find_device
     return lib_find_usb_device( $vendor, $product );
 }
 
+=item find_device_if
+
+Find a particular USB device based on the supplied predicate coderef. If
+more than one device would satisfy the predicate, the first one found is
+returned.
+
+=over 4
+
+=item pred
+
+the predicate used to select a device
+
+=back
+
+returns a device reference or undef if none was found.
+
+=cut
+
+sub find_device_if
+{
+    my $self = shift;
+    my $pred = shift;
+
+    croak( "Missing predicate for choosing a device.\n" )
+        unless defined $pred;
+
+    croak( "Predicate must be a code reference.\n" )
+        unless 'CODE' eq ref $pred;
+
+    foreach my $bus ($self->list_busses())
+    {
+        my $dev = $bus->find_device_if( $pred );
+        return $dev if defined $dev;
+    }
+
+    return;
+}
+
 =item list_devices
 
 Find all devices matching a vendor id and optional product id. If called
@@ -238,19 +332,69 @@ sub list_devices
 {
     my $self = shift;
     my ($vendor, $product) = @_;
+    my $pred = undef;
+
+    if(!defined $vendor)
+    {
+        $pred = sub { defined };
+    }
+    elsif(!defined $product)
+    {
+        $pred = sub { $vendor == $_->idVendor() };
+    }
+    else
+    {
+        $pred =
+            sub { $vendor == $_->idVendor() && $product == $_->idProduct() };
+    }
+
+    return $self->list_devices_if( $pred );
+}
+
+=item list_devices_if
+
+This method provides a more flexible interface for finding devices. It
+takes a single coderef parameter that is used to test each discovered
+device. If the coderef returns a true value, the device is returned in the
+list of matching devices, otherwise it is not.
+
+=over 4
+
+=item pred
+
+coderef to test devices.
+
+=back
+
+For example,
+
+    my @devices = $usb->list_devices_if(
+        sub { Device::USB::CLASS_HUB == $_->bDeviceClass() }
+    );
+
+Returns all USB hubs found. The device to test is available to the coderef
+in the C<$_> variable for simplicity.
+
+=cut
+
+sub list_devices_if
+{
+    my $self = shift;
+    my $pred = shift;
+
+    croak( "Missing predicate for choosing devices.\n" )
+        unless defined $pred;
+
+    croak( "Predicate must be a code reference.\n" )
+        unless 'CODE' eq ref $pred;
+
     my @devices = ();
+    local $_;
 
     foreach my $bus ($self->list_busses())
     {
-        foreach my $dev ($bus->devices())
-	{
-	    if(!defined $vendor || ($dev->idVendor() == $vendor &&
-	       (!defined $product || $product == $dev->idProduct()))
-	    )
-	    {
-	        push @devices, $dev;
-	    }
-	}
+        # Push all matching devices for this bus on list.
+        push @devices, $bus->list_devices_if( $pred );
     }
 
     return wantarray ? @devices : \@devices;
@@ -330,15 +474,18 @@ compiling the module on Windows. In theory, it should be possible to make
 the library work with LibUsb-Win32 L<http://libusb-win32.sourceforge.net/>.
 Without access to a Windows development system, I can't make those changes.
 
+The Interfaces and Endpoints are not yet proper objects. The code to extract
+this information is not yet written.
+
 =head1 ACKNOWLEDGEMENTS
 
 Thanks go to various members of the Houston Perl Mongers group for input
 on the module. But thanks mostly go to Paul Archer who proposed the project
 and helped with the development.
 
-Thanks to Josep Monés Teixidor for fixing the \C<bInterfaceClass> bug.
-Thanks to Mike McCauley for support of \C<usb_get_driver_np> and
-\C<usb_detach_kernel_driver_np>.
+Thanks to Josep Monés Teixidor for fixing the C<bInterfaceClass> bug.
+Thanks to Mike McCauley for support of C<usb_get_driver_np> and
+C<usb_detach_kernel_driver_np>.
 
 =head1 COPYRIGHT & LICENSE
 
@@ -697,6 +844,28 @@ static SV* list_interfaces( struct usb_interface* ints, unsigned count )
     return newRV_noinc( (SV*)array );
 }
 
+/*
+ * Given a pointer to a usb_config_descriptor struct, create a Perl
+ * object that contains the same data.
+ */
+static SV* build_configuration( struct usb_config_descriptor *cfg )
+{
+    HV* hash = newHV();
+    hashStoreInt( hash, "bDescriptorType", cfg->bDescriptorType );
+    hashStoreInt( hash, "wTotalLength", cfg->wTotalLength );
+    hashStoreInt( hash, "bNumInterfaces", cfg->bNumInterfaces );
+    hashStoreInt( hash, "bConfigurationValue", cfg->bConfigurationValue );
+    hashStoreInt( hash, "iConfiguration", cfg->iConfiguration );
+    hashStoreInt( hash, "bmAttributes", cfg->bmAttributes );
+    hashStoreInt( hash, "MaxPower", cfg->MaxPower*2 );
+    hashStoreSV( hash, "interfaces",
+        list_interfaces( cfg->interface, cfg->bNumInterfaces )
+    );
+
+    return sv_bless( newRV_noinc( (SV*)hash ),
+        gv_stashpv( "Device::USB::DevConfig", 1 )
+    );
+}
 
 /*
  * Given a pointer to an array of usb_config_descriptor structs, create a
@@ -705,22 +874,11 @@ static SV* list_interfaces( struct usb_interface* ints, unsigned count )
 static SV* list_configurations(struct usb_config_descriptor *cfg, unsigned count )
 {
     AV* array = newAV();
-    HV* hash = 0;
     unsigned i= 0;
 
     for(i=0; i < count; ++i)
     {
-        av_push( array, newRV_noinc( (SV*)(hash = newHV()) ) );
-        hashStoreInt( hash, "bDescriptorType", cfg[i].bDescriptorType );
-        hashStoreInt( hash, "wTotalLength", cfg[i].wTotalLength );
-        hashStoreInt( hash, "bNumInterfaces", cfg[i].bNumInterfaces );
-        hashStoreInt( hash, "bConfigurationValue", cfg[i].bConfigurationValue );
-        hashStoreInt( hash, "iConfiguration", cfg[i].iConfiguration );
-        hashStoreInt( hash, "bmAttributes", cfg[i].bmAttributes );
-        hashStoreInt( hash, "MaxPower", cfg[i].MaxPower );
-	hashStoreSV( hash, "interfaces",
-	    list_interfaces( cfg[i].interface, cfg[i].bNumInterfaces )
-	);
+        av_push( array, build_configuration( (cfg+i) ) );
     }
 
     return newRV_noinc( (SV*)array );
